/*
 * tmos - bench.c
 * 	version 0.0.0
 *
 * 	TMOS benchmark
 * 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/console.h>
#include <tmos/serial.h>
#include <tmos/stdlib.h>
#include <tmos/string.h>
#include <tmos/port.h>
#include <tmos/sched.h>
#include <asm/system.h>
#include <limits.h>

unsigned char cpu8_bench(unsigned char a) __user_preempt
{
	unsigned char i, j, k;
	unsigned char b = 1;

	for (i = 0; i < 8; i++) {
		for (j = 0; j < 128; j++) {
			for (k = 0; k < 128; k++) {
				b += a;
				a += b;
			}
		}
	}
	return a;
}

void cpu_bench(void) __user_preempt
{
	jiffies_t t;
	unsigned char result;

	printk("CPU: ");
	t = get_jiffies();
	result = cpu8_bench(2);
	t = get_jiffies() - t;
	printk("%dms (%d)\n", (unsigned int) jiffies_ms(t), result);
}

void get_jiffies_bench(void) __user_preempt
{
	jiffies_t t, t2;
	unsigned int i;

	printk("CPU: ");
	t = get_jiffies();
	for (i = 0; i < 20000; i++)
		t2 = get_jiffies();
	t = get_jiffies() - t;
	printk("%dms\n", (unsigned int) jiffies_ms(t));
}

static void dev_zero_bench(unsigned char size) __user_preempt
{
	port_t port;
	static char __user buf[32];
	unsigned int i;
	jiffies_t t;

	port = sys_port_open("/dev/zero", O_RDONLY);
	if (port == (port_t) -1)
		return;
	
	printk("/dev/zero %d ", size);
	t = get_jiffies();
//	kernel_enter();
//	preempt_disable();
	for (i = 0; i < 2048; i++) {
//			memset_d(buf, 0, size);
//		__i51_user_memset(buf, 0, size);
		port_read(port, buf, size);
	}
//	preempt_enable();
//	kernel_exit();
	t = get_jiffies() - t;
	printk("%d ms\n", (unsigned int) jiffies_ms(t));
}

void app_main(void) REENTRANT
{
//	sleep(1);
	cpu_bench();
	get_jiffies_bench();
	dev_zero_bench(1);
	dev_zero_bench(2);
	dev_zero_bench(3);
	dev_zero_bench(4);
	dev_zero_bench(6);
	dev_zero_bench(8);
	dev_zero_bench(12);
	dev_zero_bench(16);
	dev_zero_bench(20);
	dev_zero_bench(24);
	dev_zero_bench(28);
	dev_zero_bench(32);
}
