/*
 * wysmod - wysrej.c
 * 
 * Copyright (C) 2007  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/serial.h>
#include <tmos/jiffies.h>
#include <tmos/console.h>
#include <tmos/watchdog.h>

#define DATA __data

#include "display.h"

#define WYSREJ_POS

static void put_date(char base, uint8_t *date) REENTRANT
{
	c_put_hcharz_p(base + 0, date[1]);
	c_putchar_p(base + 2, '-');
	c_put_hcharz_p(base + 3, date[2]);
	c_putchar_p(base + 5, '-');
	c_put_hcharz_p(base + 6, date[3]);
}

static void print_pos(DATA uint8_t *buf) REENTRANT
{
	long pos;

	pos = buf[0] | ((unsigned short) buf[1] << 8)
			| ((long) (char) buf[2] << 16);
	if (pos < 0) {
		pos = -pos;
		c_putchar_p(11, '-');
	} else
		c_putchar_p(11, '+');
	c_put_uintz_e(18, 2, pos);
	c_putchar_p(16, '.');
#ifdef WYSREJ_FULL_POS_RANGE
	c_put_uint_e(15, 4, pos / 100);
#else
	c_put_uint_e(15, 4, (unsigned int)(pos >> 2) / 25);
#endif
	c_putchar_p(19, 'm');
}

/*
 * LCD
 *	 00000000001111111111
 * 	 01234567890123456789
 * 	"V=XX.Xm/s S=XXXX.XXm"
 * 	"XX-XX-XX    XX-XX-XX"
 */

void wysrej(void) REENTRANT
{
	code static uint8_t vel_cmd[] = { 0xff, 0x0f, 0x10, 0x00 };
	code static uint8_t pos_cmd[] = { 0xff, 0x0f, 0x11, 0x00 };
	code static uint8_t data_range_cmd[] = { 0xff, 0x0f, 0x12, 0x00 };
	uint8_t buf[12];
	char ret;
	int vel;
	
	for (;;) {
		watchdog_refresh();

		ret = ser_rxd_t(buf, 12, MS_JIFFIES(1));

		dir_tx();
		ser_tx(vel_cmd, sizeof(vel_cmd));
		dir_rx();

		ret = ser_rxd_t(buf, 2, MS_JIFFIES(200));
		dir_tx();
		if (ret != 2) {
			init_screen();
			continue;
		}
		vel = buf[0] + ((unsigned short) buf[1] << 8);

		ser_tx(pos_cmd, sizeof(pos_cmd));
		dir_rx();

		ret = ser_rxd_t(buf, 3, MS_JIFFIES(200));
		dir_tx();
		if (ret != 3) {
			init_screen();
			continue;
		}
		
		c_putchar_p(0, 'V');
		if (vel < 0) {
			vel = -vel;
			c_putchar_p(1, '-');
		} else
			c_putchar_p(1, '+');

		c_putchar_p(4, '.');
		c_putchar_p(5, d_to_char(vel % 10));
		c_put_uint_e(3, 2, vel / 10);
		c_puts_p(6, "m/s S");

		print_pos(buf);
		c_flush();

		watchdog_refresh();

		ser_tx(data_range_cmd, sizeof(data_range_cmd));
		dir_rx();

		ret = ser_rxd_t(buf, 12, MS_JIFFIES(200));
		dir_tx();
		if (ret != 12) {
			c_puts_p(LCD_COLS, "--.--.--    --.--.--");
			c_flush();
			continue;
		}

		watchdog_refresh();

		c_puts_p(LCD_COLS + 8, "    ");
		put_date(LCD_COLS + 0, buf);
		put_date(LCD_COLS + 12, buf + 6);
		c_flush();
	}
}
