/*
 * tmos - debug.c
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/serial.h>
#include <8052.h>
	
static unsigned char d_to_hex(unsigned char x)
{
	if (x >= 10)
		return x - 10 + 'a';
	return x + '0';
}
		
static void send_char_hex_ascii(unsigned char c)
{
	ser_txc(d_to_hex(c >> 4));
	ser_txc(d_to_hex(c & 0xf));
}

void ____dump_stack(unsigned char __data *sp) REENTRANT
{
	char i;

	ser_txs("Stack dump: SP=");

	/*
	 * The send_char_hex_ascii() is not reentrant. Preemption must
	 * be disabled.
	 */	
	preempt_disable();
	send_char_hex_ascii((unsigned char) sp);
	ser_txs("\n");
	for (i = 0; i > -16; i--) {
		send_char_hex_ascii(sp[i]);
		ser_txc(' ');
	}
	preempt_enable();

	ser_txs("\n--- end of stack dump ---\n");
	ser_flush();
}

void __dump_stack(unsigned long _sp) REENTRANT
{	
	unsigned char __data *sp = _sp;

	____dump_stack(sp);
}

void dump_stack(void)
{
	unsigned char __data *sp = SP;

	____dump_stack(sp);
}
