/*
 * filter-2nd filter
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * The input data comes from 12-bit ADC.
 * The output is 15-bit.
 */

#ifndef __FILTER_2ND_H
#define __FILTER_2ND_H

/*
 * number of filtered signals
 */
#ifndef F_N
#define F_N		8
#endif

#ifndef f_data
#define f_data
#endif

#ifndef FILTER_CODE
#define FILTER_CODE static void f_process(int16_t x)
#endif

#ifndef f_ch
#define f_ch 	0
#endif

static f_data int16_t f_y[F_N];
static f_data int16_t f_y1[F_N];
static f_data int8_t f_e[F_N];
	
static long y;
static int t;

static void f_stage1(void) __using ADUC_TIMER_BANK
{
#if 1
	__bit x;

	t = f_y1[f_ch] - f_y[f_ch];
	x = 0;
	if (t < 0) {
		t = -t;
		x = 1;
	}

	B = CONFIG_ADC_ADUC841_FILTER_2ND_A;	/* 3 clocks */
	/* 42 clocks */
	__asm
		mov	a, (_t + 0)		; 2
		mul	ab			; 9
		mov	(_y + 0), a		; 2
		mov	r4, b			; 2
	__endasm;

	B = CONFIG_ADC_ADUC841_FILTER_2ND_A;	/* 3 clocks */
	__asm
		mov	a, (_t + 1)		; 2
		mul	ab			; 9
		add	a, r4			; 1
		mov	(_y + 1), a		; 2
		clr	a			; 1
		mov	(_y + 3), a		; 2
		addc	a, b			; 2
		mov	(_y + 2), a		; 2
	__endasm;
	if (!x)
		y = -y;
#else
//	y = 232L * t;
#endif
}

#if 0
/*
 * time: 15 cycles
 */
static int16_t sat15(int16_t x) __using ADUC_TIMER_BANK __naked
{
	__asm
		mov	a, dph			; 2
		jb	acc.7, sat15_neg	; 4
		jb	acc.6, sat15_ovr	; 4
		ret				; 4
sat15_ovr:
		mov	dptr, #0x3fff		; 3
		ret				; 4
sat15_neg:
		jb	acc.6, sat15_ret	; 4
		mov	dptr, #0xc000		; 3
sat15_ret:
		ret				; 4
	__endasm;
}
#endif

FILTER_CODE
{
	register int t;

	f_stage1();
	y -= f_y1[f_ch];
	y += CONFIG_ADC_ADUC841_FILTER_2ND_B * x;

	/* error feedback */
	y = y - f_e[f_ch];
	f_e[f_ch] = (int8_t)(-(y & 255));

	/* division */
	y += 127;
	t = y >> 8;

	/* integrate */
	t = f_y[f_ch] + t;

	if (t > 16383) {
		t = 16383;
	} else if (t < -16384) {
		t = -16384;
	}
	f_y1[f_ch] = f_y[f_ch];
	f_y[f_ch] = t;
}

#define f_get(ch)	(f_y[ch] * 2)

#endif
