/*
 * tmos - console.c
 *
 * Copyright (C) 2007..2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tmos/kernel.h>
#include <tmos/console.h>

char d_to_char(char x)
{
	return ((x >= 10) ? 'a' - 10 : '0') + x;
}

char d_to_char_nz(char x) REENTRANT
{
	if (!x)
		return ' ';
	return d_to_char(x);
}

#if defined(CONSOLE_FB_ONLY) && defined(CONSOLE_STREAM)
#error c_putchar is unimplemented
void c_putchar(char c)
{
}

#ifndef CONSOLE_HAVE_PUTS
#error c_puts is unimplemented
void c_puts(char *c);
#endif
#endif

#ifdef CONSOLE_BLOCK

#ifndef CONSOLE_HAVE_PUTS_P
void c_puts_p(char pos, char *c) REENTRANT
{
	while (*c)
		c_putchar_p(pos++, *c++);
}
#endif

#ifndef CONSOLE_HAVE_PUTB_P
void c_putb_p(char pos, char *buf, char len) REENTRANT
{
	while (len--)
		c_putchar_p(pos++, *buf++);
	
}
#endif

void c_put_uint_e(char pos, char len, unsigned int val) REENTRANT
{
	while (len-- > 0) {
		c_putchar_p(pos--, d_to_char(val % 10));
		val /= 10;
		if (!val)
			break;
	}
	while (len-- > 0)
		c_putchar_p(pos--, ' ');
}

void c_put_uintz_e(char pos, char len, unsigned int val) REENTRANT
{
	while (len-- > 0) {
		c_putchar_p(pos--, d_to_char(val % 10));
		val /= 10;
	}
}

/*
 * c_put_hchar_p - put decimal integer
 * 	pos: end position
 * 	val: integer to print
 */
void c_put_hchar_p(char pos, unsigned char val) REENTRANT
{
	c_putchar_p(pos, d_to_char_nz(val >> 4));
	c_putchar_p(++pos, d_to_char(val & 0xf));
}

/*
 * c_put_hchar_p - put decimal integer with leading zeros
 * 	pos: end position
 * 	val: integer to print
 */
void c_put_hcharz_p(char pos, unsigned char val) REENTRANT
{
	c_putchar_p(pos, d_to_char(val >> 4));
	c_putchar_p(++pos, d_to_char(val & 0xf));
}

#endif
