/*
 * tmos - interrupts.h
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_IRQS_H
#define __ASM_IRQS_H

#include <asm/system.h>
#include <asm/jiffies.h>
#include <asm/serial.h>

#if CONFIG_SPI_ADUC841_HIGHPRIO
#define ADUC841_SPI_BANK		HIGH_PRIORITY_BANK
#else
#define ADUC841_SPI_BANK		LOW_PRIORITY_BANK
#endif

#if CONFIG_ADC_ADUC841_HIGHPRIO
#define ADUC841_ADC_BANK		HIGH_PRIORITY_BANK
#else
#define ADUC841_ADC_BANK		LOW_PRIORITY_BANK
#endif

#if CONFIG_ADC_ADUC841_TIMER_HIGHPRIO
#define TIMER1_BANK			HIGH_PRIORITY_BANK
#define TIMER2_BANK			HIGH_PRIORITY_BANK
#else
#define TIMER1_BANK			LOW_PRIORITY_BANK
#define TIMER2_BANK			LOW_PRIORITY_BANK
#endif

#define TIMER1_INTERRUPT		3
#define TIMER2_INTERRUPT		5
#define SERIAL_INTERRUPT		4
#define ADUC841_SPI_IRQ			7

void timer_irq(void) __interrupt 1 __using TIMER_BANK;
#if CONFIG_SERIAL_I51 || CONFIG_RTU_SLAVE_I51
void ser_irq(void) __interrupt SERIAL_INTERRUPT __using I51_SERIAL_BANK;
#endif
#if CONFIG_RTU_SLAVE_I51
void timer1_irq(void) __interrupt TIMER1_INTERRUPT __using TIMER1_BANK;
#endif

#ifdef CONFIG_ADC_ADUC841

#if !defined(CONFIG_SERIAL_TIMER2)
#define ADUC_TIMER_INTERRUPT	TIMER2_INTERRUPT
#define ADUC_TIMER_BANK		TIMER2_BANK
#define ADUC_TIMER2		1
#else
#define ADUC_TIMER_INTERRUPT	TIMER1_INTERRUPT
#define ADUC_TIMER_BANK		TIMER1_BANK
#define ADUC_TIMER1		1
#endif

void aduc_timer_irq(void) __interrupt ADUC_TIMER_INTERRUPT
		__using ADUC_TIMER_BANK;
#endif

#ifdef CONFIG_SPI_ADUC841
void spi_irq(void) __interrupt ADUC841_SPI_IRQ __using ADUC841_SPI_BANK;
#endif

#endif
