/*
 * tmos - trt.h
 *
 * TMOS Tiny RT subsystem
 * 
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TMOS_TRT_H
#define __TMOS_TRT_H

#include <tmos/config.h>
#include <tmos/types.h>

#if CONFIG_TRT
typedef u16 treg_t;
typedef u8 taddr_t;

#define TRT_REGS	CONFIG_TRT_REGS

#ifdef CONFIG_TRT_XDATA
#define __trt	__xdata
#else
#define __trt	__data
#endif

extern treg_t __trt __trt_regs[TRT_REGS];

#define trt_write_reg(v, i)	do { (__trt_regs[(i)]) = (v); } while (0)
#define trt_read_reg(i)		(__trt_regs[(i)])

void trt_init(void) __reentrant;
void trt_poll(void) __reentrant;

#else
#define trt_init()	do { } while (0)
#endif
	
/* TRT drivers */
void trt_jiffies_read(void) __reentrant;
void trt_adc_read(void) __reentrant;
void trt_bdi32_read(void) __reentrant;

void trt_dac_write(void) __reentrant;
void trt_bdo5_write(void) __reentrant;

#endif
